from .tensor import create_batches
from .tensor import cycle_tensor
from .tensor import expand_tail
from .tensor import expand_tail_dimensions
from .tensor import gridrange
from .tensor import iteraxis
from .tensor import move_tensor
from .tensor import move_tensor_inplace
from .tensor import next_batch
from .tensor import slices
